#define IS_NUMERIC_KEY(key) ((key >= 0x30) && (key <= 0x39))
#define IS_ALPHA_KEY(key) ((key >= 0x41) && (key <= 0x5A))
#define IS_NUMPAD_KEY(key) ((key >= 0x60) && (key <= 0x6F))
#define IS_OTHER_KEY(key) ((key == VK_SPACE) || (key == VK_BACK) || (key == VK_DELETE))

#define KEY_CHECK(key) (IS_NUMERIC_KEY(key) || IS_ALPHA_KEY(key) || IS_NUMPAD_KEY(key) || IS_OTHER_KEY(key))

#ifdef __BORLANDC__
	#define _stricmp stricmp
#endif

#if DEBUG
	#pragma comment (lib, "blackbox.lib")
#else
	#pragma comment (lib, "blackbox.lib")
	#ifndef NO_AGRESSION
//		#include <AggressiveOptimize.h>
	#endif
#endif

#include "BBApi.h"
//#include "C:\Documents and Settings\Tresni\My Documents\BB4Win\xoblite.h"
#include "BBZcmdEx.h"

extern "C" {
	DLL_EXPORT int beginPluginEx(HINSTANCE, HWND);
	DLL_EXPORT int beginPlugin(HINSTANCE);
	DLL_EXPORT void endPlugin(HINSTANCE);
	DLL_EXPORT LPCSTR pluginInfo(int);
};

LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK EditProc(HWND,UINT,WPARAM,LPARAM);

Broambox *pBBox;

int beginPluginEx(HINSTANCE hInstance, HWND hWnd)
{
	pBBox = new Broambox(hInstance, hWnd);
        strcpy(pBBox->s,pBBox->GetPluginDllPath());
        strcat(pBBox->s,"boxengine.dll");
        pBBox->h=LoadLibrary(pBBox->s);
	return pBBox->CreateBroambox();
}

int beginPlugin(HINSTANCE hInstance)
{
	return beginPluginEx(hInstance, NULL);
}

void endPlugin(HINSTANCE hInstance)
{
	SendMessage(FindWindow("About BBZCmdEx v0.1", NULL), WM_CLOSE, 0, 0);
        FreeLibrary(pBBox->h);
	pBBox->DestroyBroambox();
	delete(pBBox);
}

LPCSTR pluginInfo(int field)
{
	switch(field)
	{
		case PLUGIN_NAME: return pBBox->GetAppName();
		case PLUGIN_AUTHOR: return pBBox->GetAppAuthor();
		case PLUGIN_EMAIL: return pBBox->GetAppEmail();
		case PLUGIN_LINK: return pBBox->GetAppLink();
		case PLUGIN_RELEASE: return pBBox->GetAppRelease();
		case PLUGIN_VERSION: return pBBox->GetAppVersion();
		default:
		{
			char *temp = "";
			sprintf(temp, "%s %s (%s)", pBBox->GetAppName(), pBBox->GetAppVersion(), pBBox->GetAppRelease());
			return (LPCSTR)temp;
		}
		break;
	}
}

LRESULT CALLBACK EditProc(HWND hText,UINT msg,WPARAM wParam,LPARAM lParam)
{
	switch(msg)
	{
		case WM_SETFOCUS:
		{
			SendMessage(hText,EM_SETSEL,0,-1);
			pBBox->ToggleSlitAutoHide(true);
		}
		break;

		case WM_KILLFOCUS:
		{
//			pBBox->ResetHistoryIndex();
			pBBox->ToggleSlitAutoHide(false);
		}
		break;

		case WM_NCHITTEST:
		{
			if((GetAsyncKeyState(VK_CONTROL) & 0x8000))
				return HTCAPTION;
		}
		break;

		case WM_NCRBUTTONUP:
			return pBBox->RButtonDown(hText, msg, wParam, lParam);
		
		case WM_KEYDOWN:
		{
			if(wParam==VK_DOWN)
			{
				pBBox->HistoryPrev();
				return 0;
			}
			if(wParam==VK_SHIFT)
			{
		  //autocomplete from history when presseig UP_SHIFT
		  GetWindowText(pBBox->hEditWnd, pBBox->s, 511);
                  for(int i = pBBox->historyCount - 1; i >= 0; i--)
	           if(strstr(pBBox->history[i], pBBox->s)!=NULL && strcmp(pBBox->s, "")!=0)
			{
			SetWindowText(pBBox->hEditWnd, pBBox->history[i]);
			pBBox->historyIndex=i;
			break;
			}	
			return 0;
			}
			if(wParam==VK_UP)
			{
				pBBox->HistoryNext();
				return 0;
			}
			if(KEY_CHECK(wParam))
				InvalidateRect(hText, NULL, true);
		}
		break;

		case WM_CHAR:
		{
			if(wParam==VK_RETURN)
			{
				pBBox->Execute();
				pBBox->ToggleSlitAutoHide(false);
				return 0;

			}
		}
		break;

		case WM_CLOSE: { return 0; } break; // no ALT+F4

		//gr
		case WM_PAINT:
		{
			PAINTSTRUCT ps;
			
			HDC hdc = BeginPaint(hText, &ps);
			RECT r;
			GetClientRect(hText, &r);
			
			HDC buf = CreateCompatibleDC(NULL);
			HGDIOBJ oldbuf = SelectObject(buf, CreateCompatibleBitmap(hdc, r.right, r.bottom));
			
			pBBox->PaintBroambox(buf, &r);
			
			CallWindowProc(pBBox->wpEditProc, hText, msg, (WPARAM)buf, lParam);
			
			BitBlt(hdc, 0,0, r.right, r.bottom, buf, 0, 0, SRCCOPY);
			
			DeleteObject(SelectObject(buf, oldbuf));
			DeleteDC(buf);
			
			EndPaint(hText, &ps);
			return 0;
		}
		break;

		case WM_CTLCOLOREDIT:
		{
			//gr
			SetTextColor((HDC)wParam, pBBox->textColor);
			SetBkMode((HDC)wParam, TRANSPARENT);
			return (LRESULT)GetStockObject(NULL_BRUSH);
		}
		break;

		case WM_ERASEBKGND:
			return TRUE;

		case BB_BROADCAST:
		{
			char temp[MAX_LINE_LENGTH];
			strcpy(temp, (LPCSTR)lParam);

			if (!_stricmp(temp, "@BBShowPlugins"))
			{
				pBBox->ShowWindow();
			}
			else if(!_stricmp(temp, "@BBHidePlugins"))
			{
				pBBox->HideWindow();
			}
			if(IsInString(temp, "@Broambox"))
			{
				char broam[MAX_LINE_LENGTH], command[MAX_LINE_LENGTH], arguments[MAX_LINE_LENGTH];
				char *tokens[2];
				tokens[0] = broam;
				tokens[1] = command;

				BBTokenize(temp, tokens, 2, arguments);
				
				if(!_stricmp(command, "Toggle"))
				{
					if(!_stricmp(arguments, "Slit"))
						pBBox->ToggleInSlit();
				}
			}
		}
		return 0;

		case BB_RECONFIGURE:
		{
			pBBox->ReadSettings();
			pBBox->SizeBroambox();
			pBBox->ReplaceFont();
		}
		return 0;
	}
	return CallWindowProc(pBBox->wpEditProc,hText,msg,wParam,lParam);
}