library boxengine;

uses
  SysUtils,Classes, ShellApi, Windows, Messages;

{$R bbzres.res}

type
  TSearchEngine = record
    Identifier: string;
    Command: string;
  end;

  TAlias = record
    Alias: string;
    Command: string;
  end;

  TAnsiStrRec = packed record
    AllocSize: Longint;
    RefCount: Longint;
    Length: Longint;
  end;


const
  AnsiStrRecSize  = SizeOf(TAnsiStrRec);
  help = 'BBZCMDEX.CHM';
  AboutText = 'Copyright (c) 2004 Procic Konstantin'#13+
	      'mailto:admin_kostya@mail.ru'#13+#13+			
	      'BBZCmdEx is the command line plugin'#13+
              'for Black Box. It is based on Brian '#13+
              'Hartvigsen''s broambox and Zealot-6''s'#13+
              'ZCmd plugin for aston.'#13+#13+
              'BBZCmdEx main features'#13+#13+
              '[+] aliases for command lines '#13+
              '    (see zcmdaliases.txt)'#13+
              '[+] search engines for i-net'#13+
              '    (see zcmdengines.txt)'#13+
              '[+] internal calculator '#13+
              '    (example: "=2*(3+8)/2")'#13+
              '[+] translator to translate from'#13+
              '    one language to another'#13+
              '    (example: "?translate eng/rus/hello")'#13+
              '[+] save/load command history, no dublicate commands'#13+
              '    (see history.txt)'#13+
              '[+] autocomplete command from history on SHIFT key'#13+
              '[...] for more info type ".help"'#13+#13+
              'BBZCmdEx internal commands                    '#13+#13+
              '.about          .lock'#13+
              '.help             .run'#13+
              '.exit             .reconfig'#13+
              '.quit             .restart'#13+
              '.shutdown   .editstyle'#13+
              '.reboot        .editmenu'#13+
              '.logoff         .editplugins'#13+
              '.hibernate   .editextensions'#13+		
              '.suspend     .who'#13+
	      '.clear          .kill <app_name>';	

  AboutCaption = 'About BBZCmdEx v0.1'; //FindWindow() in destructor
  AboutIcon = 'MAINICON';

var
  setDebug: boolean;
  setDefaultText: string;
  setArgDelimit: char;
  SearchEngineFile: string;
  AliasFile: string;
  DefaultSearchEngine: pchar;
  SearchEngineList:array of TSearchEngine;
  AliasList :array of TAlias;
  cleartext: string;  // When selecting the zcommand, if the text is this, then zcommand will be cleared
  bExpanded : boolean=False;

procedure ExecCommand(cmdLine: string);forward;

//start of jedi functions
function StrSearch(const Substr, S: AnsiString; const Index: Integer): Integer; assembler;
asm
        // make sure that strings are not null

        TEST    EAX, EAX
        JZ      @@SubstrIsNull

        TEST    EDX, EDX
        JZ      @@StrIsNull

        // limit index to satisfy 1 <= index, and dec it

        DEC     ECX
        JL      @@IndexIsSmall

        // ebp will hold # of chars in Substr to compare, esi pointer to Str,
        // edi pointer to Substr, ebx primary search char

        PUSH    EBX
        PUSH    ESI
        PUSH    EDI
        PUSH    EBP

        // set the AnsiString pointers

        MOV     ESI, EDX
        MOV     EDI, EAX

        // save the (Index - 1) in edx

        MOV     EDX, ECX

        // save the address of Str to compute the result

        PUSH    ESI

        // temporary get the length of Substr and Str

        MOV     EBX, [EDI-AnsiStrRecSize].TAnsiStrRec.Length
        MOV     ECX, [ESI-AnsiStrRecSize].TAnsiStrRec.Length

        // dec the length of Substr because the first char is brought out of it

        DEC     EBX
        JS      @@NotFound

        // # of positions in Str to look at = Length(Str) - Length(Substr) - Index - 2

        SUB     ECX, EBX
        JLE     @@NotFound

        SUB     ECX, EDX
        JLE     @@NotFound

        // point Str to Index'th char

        ADD     ESI, EDX

        // # of chars in Substr to compare

        MOV     EBP, EBX

        // clear EAX & ECX (working regs)

        XOR     EAX, EAX
        XOR     EBX, EBX

        // bring the first char out of the Substr, and
        // point Substr to the next char

        MOV     BL, [EDI]
        INC     EDI

        // jump into the loop

        JMP     @@Find

@@FindNext:

        // update the loop counter and check the end of AnsiString.
        // if we reached the end, Substr was not found.

        DEC     ECX
        JL      @@NotFound

@@Find:

        // get current char from the AnsiString, and /point Str to the next one.
        MOV     AL, [ESI]
        INC     ESI

        // does current char match primary search char? if not, go back to the main loop

        CMP     AL, BL
        JNE     @@FindNext

        // otherwise compare SubStr

@@Compare:

        // move # of char to compare into edx, edx will be our compare loop counter.

        MOV     EDX, EBP

@@CompareNext:

        // check if we reached the end of Substr. If yes we found it.

        DEC     EDX
        JL      @@Found

        // get last chars from Str and SubStr and compare them,
        // if they don't match go back to out main loop.

        MOV     AL, [EDI+EDX]
        CMP     AL, [ESI+EDX]
        JNE     @@FindNext

        // if they matched, continue comparing

        JMP     @@CompareNext

@@Found:
        // we found it, calculate the result and exit.

        MOV     EAX, ESI
        POP     ESI
        SUB     EAX, ESI

        POP     EBP
        POP     EDI
        POP     ESI
        POP     EBX
        RET

@@NotFound:
        // not found it, clear result and exit.

        XOR     EAX, EAX
        POP     ESI
        POP     EBP
        POP     EDI
        POP     ESI
        POP     EBX
        RET

@@IndexIsSmall:
@@StrIsNull:
        // clear result and exit.

        XOR     EAX, EAX

@@SubstrIsNull:
@@Exit:
end;


function StrCharCount(const S: AnsiString; C: AnsiChar): Integer;
var
  I: Integer;
begin
  Result := 0;
  for I := 1 to Length(S) do
    if S[I] = C then
      Inc(Result);
end;

function StrStrCount(const S, SubS: AnsiString): Integer;
var
  I: Integer;
begin
  Result := 0;
  if (Length(SubS) > Length(S)) or (Length(SubS) = 0) or (Length(S) = 0) then
    Exit;
  if Length(SubS) = 1 then
  begin
    Result := StrCharCount(S, SubS[1]);
    Exit;
  end;
  I := StrSearch(SubS, S, 1);

  if I > 0 then
    Inc(Result);

  while (I > 0) and (Length(S) > I+Length(SubS)) do
  begin
    I := StrSearch(SubS, S, I+1);

    if I > 0 then
      Inc(Result);
  end
end;
// end jedi functions

//  ,  cmd
procedure killapp(cmd:string);
var hwnd:THandle;
    str :array[0..MAX_PATH]of char;
    s:string;
    len:cardinal;	
begin 
 len := lstrlen(PChar(cmd));
 if(len>6)then 
  s := copy(cmd, 6 , len) else s:='';
if s<>'' then 
begin
hwnd:=GetDesktopWindow();
hwnd:=GetWindow(hwnd, GW_CHILD);
 while (hwnd <>0)do 
 begin
  GetWindowText(hwnd,str,128);
    if(StrStrCount(UpperCase(str), UpperCase(s))<>0)then
      PostMessage(hwnd,WM_CLOSE,0,0);

    hwnd:=GetWindow(hwnd, GW_HWNDNEXT);
 end;
end;
end;

procedure AboutDialog;
//  MessageBoxIndirect,     
//   About     Windows 9x  NT
var
  Version: TOSVersionInfo;
  MsgBoxParamsW: TMsgBoxParamsW;
  MsgBoxParamsA: TMsgBoxParamsA;
Begin
  Version.dwOSVersionInfoSize := SizeOf(TOSVersionInfo);
  GetVersionEx(Version);
  if Version.dwPlatformId = VER_PLATFORM_WIN32_NT then begin
    FillChar(MsgBoxParamsW, SizeOf(MsgBoxParamsW), #0);
    with MsgBoxParamsW do begin
      cbSize := SizeOf(MsgBoxParamsW);
      hwndOwner := GetDesktopWindow;
      hInstance := SysInit.hInstance;
      lpszText  := AboutText;
      lpszCaption := AboutCaption;
      lpszIcon := AboutIcon;
      dwStyle := MB_USERICON;
    end;
    MessageBoxIndirectW(MsgBoxParamsW);
  end else begin
    FillChar(MsgBoxParamsA, SizeOf(MsgBoxParamsA), #0);
    with MsgBoxParamsA do begin
      cbSize := SizeOf(MsgBoxParamsA);
      hwndOwner := GetDesktopWindow;
      hInstance := SysInit.hInstance;
      lpszText  := AboutText;
      lpszCaption := AboutCaption;
      lpszIcon := AboutIcon;
      dwStyle := MB_USERICON;
    end;
    MessageBoxIndirectA(MsgBoxParamsA);
  end;
end;

procedure AstonMB(s:string);
begin
 MessageBox(0,PChar(s),'',MB_OK);
end;

//return program directory
Function DefaultDir:string;
var i:integer;
    TheFileName:array [0..MAX_PATH] of char;
begin
 FillChar(TheFileName, sizeof(TheFileName), #0);
 GetModuleFileName(hInstance, TheFileName, sizeof(TheFileName));
 for i:= lstrlen(TheFileName) downto 0 do
 if TheFileName[i]='\' then break;
 while(i<=lstrlen(TheFileName))do begin
 TheFileName[i]:=#0;
 inc(i);
 end;
 ChDir(TheFileName);
 DefaultDir:=TheFileName;
end;

{----------------------------------------------------------------}

//////////////////////////////////////////////////////////////////
//   function txt                                               //
//                                                              //
//   formats text as lowercase,                                 //
//   and trims leading and trailing spaces.                     //
//   Used for comparissons                                      //
//   Returns true if sucessful, false if file cannot be found   //
//////////////////////////////////////////////////////////////////
function txt(input: string): string;
begin
  result := lowercase(trim(input));
end;

//////////////////////////////////////////////////////////////////
//   function SearchEngineInit                                  //
//                                                              //
//   Builds the list of search engines                          //
//   Returns true if sucessful, false if file cannot be found   //
//////////////////////////////////////////////////////////////////
function SearchEngineInit(): bool;
var
  tslFile: TStringList;
  i, i2: integer;

begin
  try
    tslFile := tStringList.Create;
    tslFile.LoadFromFile(SearchEngineFile);
  except  //  filenotfound
    result := False;
    exit;
  end;

  i2 := 0;
  SetLength(SearchEngineList, tslFile.Count);
  for i := 0 to tslFile.Count -1 do                                                                             
  begin
    if txt(copy(tslFile[i], 0, pos(' ', tslFile[i])-1)) = 'default' then
    begin
      DefaultSearchEngine := pchar(txt(copy(tslFile[i], pos(' ', tslFile[i]), strlen(pchar(tslFile[i]))-pos(' ', tslFile[i])+1)));
    end
    else if copy(tslFile[i], 0, 1) <> ';' then
    begin
      SearchEngineList[i2].Identifier := txt(copy(tslFile[i], 1, pos(' ', tslFile[i])));
      SearchEngineList[i2].Command := copy(tslFile[i], pos(' ', tslFile[i]), strlen(pchar(tslFile[i]))-pos(' ', tslFile[i])+1);
      i2  := i2 + 1;
    end;
  end;
  result := true;
end;

//////////////////////////////////////////////////////////////////
//   function AliasListInit                                     //
//                                                              //
//   Builds the list of Aliases                                 //
//   Returns true if sucessful, false if file cannot be found   //
//////////////////////////////////////////////////////////////////
function AliasListInit(): bool;
var
  tslFile: TStringList;
  i, i2: integer;
begin
  try
    tslFile := tStringList.Create;
    tslFile.LoadFromFile(AliasFile);
  except
    //   filenotfound
    result := False;
    exit;
  end;

  i2 := 0;
  SetLength(AliasList, tslFile.Count);
  for i := 0 to tslFile.Count -1 do
  begin
    if copy(tslFile[i], 0, 1) <> ';' then
    begin
      AliasList[i2].Alias := txt(copy(tslFile[i], 1, pos(' ', tslFile[i])));
      AliasList[i2].Command := trim(copy(tslFile[i], pos(' ', tslFile[i]), strlen(pchar(tslFile[i]))-pos(' ', tslFile[i])+1));
      i2  := i2 + 1;
    end;
  end;
  result := true;
end;

//////////////////////////////////////////////////////////////////
//   function GetSearchAddress                                  //
//                                                              //
//   Returns the search engine URL for the specified            //
//      search engine                                           //
//   If SearchEngineID is '' then returns the default engine    //
//////////////////////////////////////////////////////////////////
function GetSearchAddress(SearchEngineID:string): string;
var
  i: integer;
begin
  // if search engine is not specified, return URL for default search engine
  if strlen(pchar(SearchEngineID)) = 0 then
  begin
    try
     result := SearchEngineList[0].Command;
    except
     result := '';
    end;
  end

  // else, look up specified search engine
  else
  begin
    try
      for i := 0 to length(SearchEngineList) - 1 do
      begin
        if txt(SearchEngineID) = txt(SearchEngineList[i].Identifier) then
        begin
          result := SearchEngineList[i].Command;
          exit;
        end;
      end;
    except
      result := '';
    end;
  end;
end;

//////////////////////////////////////////////////////////////////
//   function CalcExpression                                    //
//                                                              //
//   Calculator... evaluates a string expression                //
//   Returns result of evaluation                               //
//////////////////////////////////////////////////////////////////
function CalcExpression (const expression : string) : extended;
var
  pos : Integer;
  ch : char;

  function GetChar : char;
  begin
    if pos < Length (expression) then
    begin
      Inc (pos);
      ch := expression [pos]
    end
    else ch := #0;
    result := ch
  end;

  function GetNonWhitespace : char;
  begin
    repeat
      GetChar
    until not (ch in [' ', #9]);
    result := ch
  end;

  function CalcAddition : extended; forward;

  function CalcNumber : extended;
  var
    s : string;
    gotPoint : boolean;
  begin
    gotPoint := False;
    while ch in ['0'..'9'] do
    begin
      s := s + ch;
      if (GetChar = '.') and not gotPoint then
      begin
        s := s + ch;
        GetChar;
        gotPoint := True
      end
    end;
    result := StrToFloat (s)
  end;

  function CalcExpressionInBrackets : double;
  begin
    result := CalcAddition;
    if ch = ')' then
      GetChar
    else
      raise Exception.Create ('Missing '')'' in expression')
  end;

  function CalcTerm : extended;
  begin
    case ch of
      '0'..'9' : result := CalcNumber;
      '(' : result := CalcExpressionInBrackets;
      else raise Exception.Create ('Syntax error in expression')
    end
  end;

  function CalcSignedTerm : extended;
  begin
    case GetNonWhitespace of
      '+' : result := CalcSignedTerm;
      '-' : result := -CalcSignedTerm;
      else result := CalcTerm
    end
  end;

  function CalcPower : extended;
  begin
    result := CalcSignedTerm;
    while ch = '^' do
      result := exp (ln (result) * CalcSignedTerm)
  end;

  function CalcMultiplication : extended;
  begin
    result := CalcPower;
    while ch in ['*', '/'] do
      case ch of
        '*' : result := result * CalcPower;
        '/' : result := result / CalcPower
      end
  end;

  function CalcAddition : extended;
  begin
    result := CalcMultiplication;
    while ch in ['+', '-'] do
      case ch of
        '+' : result := result + CalcMultiplication;
        '-' : result := result - CalcMultiplication
      end
  end;

begin
  pos := 0;
  result := CalcAddition
end;

//////////////////////////////////////////////////////////////////
//   function GetArgs                                           //
//                                                              //
//   Parses a command line                                      //
//   Returns only the arguments in command line                 //
//////////////////////////////////////////////////////////////////

function GetArgs(cmdLine:string): string;
var
 argstart: integer;
begin
   // argstart := 0;
    cmdLine := pchar(trim(cmdLine));
    {
    if copy(cmdline, 0 , 1) = '"' then
     begin
      argstart := pos('"', s);
     end
    else
     }
      argstart := pos(' ', cmdLine);


    if argstart = 0 then  // No arguments
    begin
      result := '';
      exit;
    end
    else  // Arguments
    begin
      result := trim(copy(cmdLine, argstart + 1, strlen(pchar(cmdLine)) - argstart));
    end;
end;


//////////////////////////////////////////////////////////////////
//   function RemoveArgs                                        //
//                                                              //
//   Parses a command line                                      //
//   Returns only the command in command line                   //
//////////////////////////////////////////////////////////////////
function RemoveArgs(cmdLine:string): string;
var
  argstart: integer;
begin
  //argstart := 0;
  cmdLine := pchar(trim(cmdLine));
  {
  if copy(cmdline, 0 , 1) = '"' then
  begin
    argstart := pos('"', s);
  end
  else
  }
    argstart := pos(' ', cmdLine);

  if argstart = 0 then  // No arguments
  begin
    result := cmdLine;
    exit;
    end
    else  // Arguments
    begin
      result := trim(copy(cmdLine, 1, argstart));
    end;
end;
/////


//////////////////////////////////////////////////////////////////
//   function GetAlias                                          //
//                                                              //
//   Searched alias array for a valid alias.                    //
//   Returns the command if sucessful,                          //
//   Returns original text if unsucessful                       //
//////////////////////////////////////////////////////////////////
function GetAlias(alias: string): string;
var
  i: integer;
begin
  alias := txt(alias);
  for i := 0 to Length(AliasList) - 1 do
  begin
    if alias = AliasList[i].Alias then
    begin
      result := AliasList[i].Command;
      exit;
    end;
  end;
  result := alias;
end;


//////////////////////////////////////////////////////////////////
//   function ExecCommand                                       //
//                                                              //
//   Executes a command line                                    //
//   if command line begins with ., treats it as a command      //
//   if command line begins with ?, launches an internet search //
//   if command line begins with =, evaluate expression         //
//   otherwise, launch program or website                       //
//////////////////////////////////////////////////////////////////
procedure ExecCommand(cmdLine: string);
var
  cmd: string;
  args: string;
  res: pchar;
  seID: string;
  shellret: integer;
  SLarg: TStringList;
  stemp: string;
  i: integer;

begin
  if lstrlen(pchar(cmdLine))<1 then exit;

  // Help
  if copy(txt(cmdline), 0, 5) = '.help' then
  begin
    ShellExecute(0, 'open',pchar(DefaultDir+'\'+help),nil, nil, SW_SHOW);
  end
  
  // About Dialog
  else if copy(cmdline, 0 , 6) = '.about' then
  begin
   AboutDialog;
  end

  // .kill app by string
  else if copy(cmdline, 0 , 5) = '.kill' then
  begin
   KillApp(cmdline);
  end

  // Internet Search
  else if copy(cmdline, 0 , 1) = '?' then
  begin
    ////////////////////////
    // Default Search Engine
    ////////////////////////
    if copy(cmdLine, 2, 1) = ' ' then
    begin
      cmd := GetSearchAddress('');
      if strlen(pchar(cmd)) = 0 then
      begin
        AstonMB('Error: No Default Search Engine');
        exit;
      end;
    end
{    else if copy(txt(cmdline), 2, 4) = 'site' then  // Search a site
    begin
      cmdline := trim(cmdline);
      site := copy(cmdline, 7, pos(' ', copy(cmdline, 7, strlen(pchar(cmdline)))));

      args := copy(cmdline, 7 + strlen(pchar(site)), strlen(pchar(cmdline)));
      args := stringreplace(args, ' ', '+', [rfReplaceAll]);

      cmd := 'http://www.google.com/search?q=<args>+site:<site>';
      cmd := stringreplace(cmd, '<site>', site, [rfReplaceAll]);
      cmd := stringreplace(cmd, '<args>', args, [rfReplaceAll]);
      cmd := trim(cmd);
    end
}
    //////////////////////////
    // Specify a Search Engine
    //////////////////////////
    else
    begin
      seID := copy(cmdLine, 2, pos(' ', cmdLine)-2);
      seID := trim(seID);
      cmd := GetSearchAddress(seID);
      if strlen(pchar(cmd)) = 0 then
      begin
        AstonMB('Error: Unknown Search Engine: ' + seID);
        exit;
      end;
    end;

    // Set args string to everything after the first space
    args := trim(copy(cmdLine, pos(' ', cmdLine), strlen(pchar(cmdLine)) - pos(' ', cmdLine)+1));

    // Replace all spaces with +'s
    args := stringreplace(args, ' ', '+', [rfReplaceAll]);

    stemp := args;
    while Pos(setArgDelimit, stemp)>0 do stemp[Pos(setArgDelimit, stemp)]:=#10;
    stemp := adjustlinebreaks(stemp);
    SLarg := TStringList.Create;
    SLarg.Text := stemp;

    try
      cmd := stringreplace(cmd, '<args>', args, [rfReplaceAll]);
    except end;

    for i := 0 to SLarg.Count-1 do
    begin
      try
        cmd := stringreplace(cmd, '<arg' + inttostr(i + 1) + '+>', copy(args, pos(SLarg[i], args), strlen(pchar(args))), [rfReplaceAll]);
      except
        cmd := stringreplace(cmd, '<arg' + inttostr(i +1) + '+>', '', [rfReplaceAll]);
      end;

      try
        cmd := stringreplace(cmd, '<arg' + inttostr(i+1) + '>', SLarg[i], [rfReplaceAll]);
      except
        cmd := stringreplace(cmd, '<arg' + inttostr(i+1) + '>', '', [rfReplaceAll]);
      end;
    end;

    cmd := trim(cmd);
    SLarg.Free;

    if setDebug then
      AstonMB('-Debug-'#13#10+'Command Line: ' + cmdLine + #13#10 + 'Cmd: ' + cmd);

    shellret := ShellExecute(0,'open',pchar(cmd),'',nil,SW_SHOWDEFAULT);
    if shellret < 32 then AstonMB('Error occured while launching internet search');
  end

  // Calculator
  else if copy(cmdline, 0 , 1) = '=' then
  begin
    args := pchar(copy(cmdLine, 2, strlen(pchar(cmdLine)) - 1));
    res := pchar(FloatToStrF(CalcExpression(args), ffGeneral	, 15, 14));
   //    Form1.ComboBox1.Text := res;
   //SetWindowText(FindWindow('Edit','ZCmdEditName'), res);
   MessageBox(0,res,'Calculator',MB_OK);
    ClearText := res;
    exit;
  end

  // Either an Alias or an executable
  else
  begin
    // Get the typed arguments
    args := GetArgs(cmdLine);
    // Get the typed command
    cmd := RemoveArgs(cmdLine);
    // was the typed command an alias?
    cmd := GetAlias(cmd);
    // add alias arguments to typed arguments
    args := GetArgs(cmd) + args;
    // strip arguments from the cmd
    cmd := RemoveArgs(cmd);

    if setDebug then
      AstonMB('-Debug-'#13#10+'Command Line: ' + cmdLine + #13#10 + 'Cmd: ' + cmd + #13#10 + 'Args: ' + args);

    shellret := ShellExecute(0,'open',pchar(cmd),pchar(args),nil,SW_SHOWDEFAULT);

    if shellret < 32 then AstonMB('Error: ' + SysErrorMessage(GetLastError));

  end;
 // Form1.ComboBox1.Text := setDefaultText;
 // SetWindowText(FindWindow('Edit','ZCmdEditName'),PChar(setDefaultText));
end;

procedure EC();stdcall;
var
 sPath:array [0..MAX_PATH]of char;
begin
  setDebug := FALSE;
  setArgDelimit := '/';
  setDefaultText := '';
  GetModuleFileName(hinstance, sPath, sizeof(sPath));
  SearchEngineFile := extractfilepath(spath) + 'zcmdengines.txt';
  AliasFile := extractfilepath(spath) + 'zcmdaliases.txt';
  if not SearchEngineInit() then AstonMB('Search engine not loaded');
  if not AliasListInit()then AstonMB('Alias engine not loaded');
  GetWindowText(FindWindow('Edit',nil),sPath,MAX_PATH-1);
  ExecCommand(sPath);
end;

exports EC;
end.
 
